@@ -62,13 +62,20 @@ cal_home_advance_date_by_time_period(struct tm* start,
   return mktime(start);
 }
 
-static void
+void
 cal_home_time_today_midnight(struct tm* today)
 {
   time_get_local(today);
   today->tm_hour = today->tm_min = today->tm_sec = 0;
 }
 
+static void
+cal_home_time_today_now(struct tm* today)
+{
+  time_get_local(today);
+  today->tm_sec = 0;
+}
+
 static int
 cal_row_count_for_cal_size(const gint& cal_size)
 {
@@ -240,12 +247,17 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
 					   gsize up_to)
 {
   struct tm end_date;
-  cal_home_time_today_midnight(&end_date);
+  struct tm end_date_x;
+  cal_home_time_today_now(&end_date);
+  cal_home_time_today_midnight(&end_date_x);
+  end_date_x.tm_mday++;
+  time_t end_time_x = mktime(&end_date_x);
   time_t end_time = cal_home_advance_date_by_time_period(&end_date,
 							 desktop_plugin->cal_time_period);
-
+  time_t date_today_time = mktime(&desktop_plugin->date_today);
   GtkTreeIter iter;
-  
+  int earliest_update_end_time = end_time_x;
+  desktop_plugin->today_events = 0;
   for(SortedCComponentSet::const_iterator event_iter = ccomponent_set.begin();
       event_iter != ccomponent_set.end() && up_to>0;
       ++event_iter)
@@ -259,9 +271,17 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
     struct tm* time = localtime(&start);
     char buf[255];
     if(cal_date_on_today((*event_iter)->getDateStart(), desktop_plugin) || 
-       start < mktime(&desktop_plugin->date_today))
+       start < date_today_time)
     {
-      if((*event_iter)->getAllDay() || start < mktime(&desktop_plugin->date_today))
+      ++desktop_plugin->today_events;
+      
+      if(earliest_update_end_time>(*event_iter)->getDateEnd())
+	earliest_update_end_time = (*event_iter)->getDateEnd()+60;
+      if(date_today_time < (*event_iter)->getDateStart() &&
+	 earliest_update_end_time > (*event_iter)->getDateStart())
+	earliest_update_end_time = (*event_iter)->getDateStart()+60;
+
+      if((*event_iter)->getAllDay() || start < date_today_time)
       {      
 	gtk_list_store_set(list_store, &iter,
 			   0, dgettext("calendar", "cal_va_allday_home"),
@@ -269,6 +289,7 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
       }
       else
       {
+	
 	if(desktop_plugin->has_24_time_format)
 	  strftime(buf, 255,dgettext("hildon-libs","wdgt_va_24h_time"), time);
 	else
@@ -304,6 +325,14 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
     g_object_unref(pixbuf);
     --up_to;
   }
+  struct tm now;
+  cal_home_time_today_now(&now);
+  
+  if(desktop_plugin->time_out_handler)
+  {
+    g_source_remove(desktop_plugin->time_out_handler);
+  }
+  desktop_plugin->time_to_update = earliest_update_end_time;
 }
 
 static void
@@ -387,7 +416,7 @@ cal_read_ccomponents_all_visible(SortedCComponentSet& all_components,
 void 
 cal_home_load_db_events(CalHomePlugin* desktop_plugin)
 {
-  cal_home_time_today_midnight(&desktop_plugin->date_today);
+  cal_home_time_today_now(&desktop_plugin->date_today);
   gtk_list_store_clear(desktop_plugin->event_list);
   SortedCComponentSet all_components;
   cal_read_ccomponents_all_visible(all_components,
