@@ -108,7 +108,6 @@ show_visible_calendars_dlg(GtkButton* button, gpointer data)
   GtkWidget* pan = hildon_pannable_area_new();
   hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pan), list_box);
   gtk_box_pack_start(GTK_BOX(content), pan, TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(content), list_box, TRUE, TRUE, 0);
   gtk_window_set_default_size(GTK_WINDOW(dialog), -1, 325);  
   gtk_widget_show_all(dialog);
 
@@ -170,11 +169,23 @@ cal_show_settings_dialog(GtkWidget* widget, gpointer data)
   hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
 				    "Auto");
   hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
-				    dgettext("osso-sharing-ui", "share_fi_options_size_large"));
+				    "1");
   hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
-				    dgettext("osso-sharing-ui", "share_fi_options_size_medium"));
+				    "2");
   hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
-				    dgettext("osso-sharing-ui", "share_fi_options_size_small"));
+				    "3");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "4");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "5");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "6");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "7");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "8");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				    "9");
 
   hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(cal_time_selector),
 				    "1 Week");
@@ -231,9 +242,13 @@ cal_show_settings_dialog(GtkWidget* widget, gpointer data)
 
   hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(open_mode_button),
 				    HILDON_TOUCH_SELECTOR(launch_view_selector));
+  if(desktop_plugin->cal_row_count==-1)
+    hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				     0, 0);
+  else
+    hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(cal_size_selector),
+				     0, desktop_plugin->cal_row_count);
   
-  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(cal_size_selector),
-				   0, desktop_plugin->cal_size);
 
   hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(cal_size_button),
 				    HILDON_TOUCH_SELECTOR(cal_size_selector));
@@ -268,7 +283,11 @@ cal_show_settings_dialog(GtkWidget* widget, gpointer data)
     desktop_plugin->visible_cals = (gint*)g_memdup(settings_data->visible_cals, 
 						   sizeof(gint)*settings_data->number_visible_cals);
     desktop_plugin->launch_view = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(launch_view_selector), 0) + 1;
-    desktop_plugin->cal_size = CalSize(hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(cal_size_selector), 0));
+    int row_count = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(cal_size_selector), 0);
+    if(row_count == 0)
+      desktop_plugin->cal_row_count = -1;
+    else
+      desktop_plugin->cal_row_count = row_count;
     desktop_plugin->cal_time_period = CalTimePeriod(hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(cal_time_selector), 0));
     cal_home_save_settings(desktop_plugin);
     cal_home_load_db_events(desktop_plugin);
