@@ -28,11 +28,11 @@
 
 #include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
+#include <gconf/gconf-client.h>	
 
 #include <CMulticalendar.h>
 #include <CalInterface.h>
 #include <CComponent.h>
-#include <gconf/gconf-client.h>	
 
 #include "cal-home-widget.h"	
 #include "cal-home-load-and-store.h"	
@@ -46,7 +46,7 @@ extern "C" {
 
 
 #define CAL_BG_IMAGE_PATH "/usr/share/icons/hicolor/64x64/hildon/calendar_applet.png"
-#define GCONF_TIME_FORMAT_PATH "/apps/clock/time-format"
+#define GCONF_TIME_FORMAT_PATH "/apps/clock"
 
 static const char * 	CALENDAR_DBUS_LISTENER_SIGNAL = "dbChange";
 
@@ -58,15 +58,16 @@ static const int CAL_WIDGET_WIDTH = 352;
 void
 cal_widget_resize_for_cal_size(CalHomePlugin* desktop_plugin)
 {
-  gint row_count = desktop_plugin->cal_row_count;
-  if(row_count == -1)
-    row_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(desktop_plugin->event_list), NULL); 
+  gint cal_row_count = desktop_plugin->cal_row_count;
+  gint row_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(desktop_plugin->event_list), NULL);
+  if(cal_row_count == -1)
+    cal_row_count = row_count;
   gtk_widget_set_size_request(desktop_plugin->event_list_container, 
 			      CAL_WIDGET_WIDTH, 
 			      CAL_WIDGET_ENTRY_ROW_HEIGHT*row_count);
   gtk_window_resize(GTK_WINDOW (desktop_plugin), 
 		    CAL_WIDGET_WIDTH, 
-		    (CAL_WIDGET_ENTRY_ROW_HEIGHT*row_count)+CAL_WIDGET_TITLE_HEIGHT+CAL_WIDGET_BOTTON_HEIGHT);
+		    (CAL_WIDGET_ENTRY_ROW_HEIGHT*cal_row_count)+CAL_WIDGET_TITLE_HEIGHT+CAL_WIDGET_BOTTON_HEIGHT);
 }
 
 static gboolean
@@ -74,11 +75,11 @@ cal_24_time_format()
 {
   GConfClient* client = NULL;
   client = gconf_client_get_default();
-  if(!GCONF_IS_CLIENT(client))
-    return TRUE;
+  if(!GCONF_IS_CLIENT(client)) 
+    return TRUE; 
   
   gboolean is24Format = TRUE;
-  is24Format = gconf_client_get_bool(client, GCONF_TIME_FORMAT_PATH, NULL);
+  is24Format = gconf_client_get_bool(client, GCONF_TIME_FORMAT_PATH "/time-format", NULL);
   g_object_unref(client);
   return is24Format;
 }
@@ -161,30 +162,43 @@ render_day_on_title(CalHomePlugin* desktop_plugin)
   cairo_destroy(cr);
 }
 
-static gboolean
-cal_update_time(gpointer data);
 
 static void
 register_time_out_handler(CalHomePlugin* desktop_plugin)
 {
-  static int seconds_in_day = 24*60*60;
-  struct tm now;
-  time_get_local(&now);
-  desktop_plugin->time_out_handler = g_timeout_add_seconds(seconds_in_day-(now.tm_hour*3600 + 60 * now.tm_min + now.tm_sec),
+  if(desktop_plugin->time_out_handler)
+  {
+    g_source_remove(desktop_plugin->time_out_handler);
+    desktop_plugin->time_out_handler = 0;
+  }
+  time_t now = time_get_time();
+  int delay = desktop_plugin->time_to_update - now;
+  desktop_plugin->time_out_handler = g_timeout_add_seconds(delay,
 							   cal_update_time, 
 							   (gpointer) desktop_plugin);
 }
 
+static void
+calendar_update_view(CalHomePlugin* desktop_plugin)
+{
+  render_day_on_title(desktop_plugin);
+  cal_home_load_db_events(desktop_plugin);
+  cal_widget_resize_for_cal_size(desktop_plugin);
+  register_time_out_handler(desktop_plugin);
+}
+
 static void
 cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin);
 
-static gboolean
+gboolean
 cal_update_time(gpointer data)
 {
   if(CAL_HOME_IS_HOME_PLUGIN(data))
   {
     CAL_HOME_PLUGIN(data)->update_view =  TRUE;
+    gdk_threads_enter();
     cal_update_if_visible_and_date_changed(CAL_HOME_PLUGIN(data));
+    gdk_threads_leave();
     register_time_out_handler((CalHomePlugin*)data);
   }
   return FALSE;
@@ -198,39 +212,41 @@ handle_time_format_changed(GConfClient* client,
 {
   g_return_if_fail(CAL_HOME_IS_HOME_PLUGIN(user_data));
   CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(user_data);
-  desktop_plugin->has_24_time_format = cal_24_time_format();
+  desktop_plugin->has_24_time_format = gconf_client_get_bool(client, GCONF_TIME_FORMAT_PATH "/time-format", NULL);
   desktop_plugin->update_view = TRUE;
+  gboolean on;
+  g_object_get(GTK_WIDGET(desktop_plugin), "is-on-current-desktop", &on, NULL);
+  if(on)
+  {
+    gdk_threads_enter();
+    calendar_update_view(desktop_plugin);
+    gdk_threads_leave();
+  }
 }
 
 static gboolean 
-today_date_changed(CalHomePlugin* desktop_plugin)
+time_to_update(CalHomePlugin* desktop_plugin)
 {
-  struct tm now;
-  time_get_local(&now);
-  gboolean time_differ = 
-    (now.tm_mday != desktop_plugin->date_today.tm_mday ||
-     now.tm_mon != desktop_plugin->date_today.tm_mon ||
-     now.tm_year != desktop_plugin->date_today.tm_year);
-
-  return time_differ;
+  time_t now = time_get_time();
+  if((desktop_plugin->time_to_update - now)<=0)
+    return TRUE;
+  else
+    return FALSE;
 }
 
 static void
 cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin)
 {
-  if(!today_date_changed(desktop_plugin))
+  if(!time_to_update(desktop_plugin))
   {
     return;
   }
   gboolean on;
   g_object_get(GTK_WIDGET(desktop_plugin), "is-on-current-desktop", &on, NULL);
 
-  
   if(on)
   {
-    render_day_on_title(desktop_plugin);
-    cal_home_load_db_events(desktop_plugin);
-    cal_widget_resize_for_cal_size(desktop_plugin);
+    calendar_update_view(desktop_plugin);
   }
   else
   {
@@ -241,7 +257,18 @@ cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin)
 static void
 handle_time_changed(DBusGProxy *object, gint time, CalHomePlugin* desktop_plugin)
 {
-  cal_update_if_visible_and_date_changed(desktop_plugin);
+  gboolean on;
+  g_object_get(desktop_plugin, "is-on-current-desktop", &on, NULL);
+  if(on)
+  {
+    gdk_threads_enter();
+    calendar_update_view(desktop_plugin);
+    gdk_threads_leave();
+  }
+  else
+  {
+    CAL_HOME_PLUGIN(desktop_plugin)->update_view = TRUE;
+  }
 }
 
 static DBusHandlerResult
@@ -258,9 +285,9 @@ handle_db_change(DBusConnection *bus,
       g_object_get(desktop_plugin, "is-on-current-desktop", &on, NULL);
       if(on)
       {
-	render_day_on_title(desktop_plugin);
-	cal_home_load_db_events(desktop_plugin);
-	cal_widget_resize_for_cal_size(desktop_plugin);
+	gdk_threads_enter();
+	calendar_update_view(desktop_plugin);
+	gdk_threads_leave();
       }
       else
       {
@@ -275,23 +302,20 @@ static void
 cal_on_current_desktop(GtkWidget* widget,
 		       gpointer data)
 {
-  g_return_if_fail(CAL_HOME_IS_HOME_PLUGIN(widget));
-
   CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(widget);
+
   gboolean on;
   g_object_get(widget, "is-on-current-desktop", &on, NULL);
   if(on)
   {
-    if(!desktop_plugin->time_out_handler)
-    {
-      register_time_out_handler(desktop_plugin);
-    }
-    if(today_date_changed(desktop_plugin) ||
+    if(time_to_update(desktop_plugin) ||
        desktop_plugin->update_view)
     {
-      render_day_on_title(desktop_plugin);
-      cal_home_load_db_events(desktop_plugin);
-      cal_widget_resize_for_cal_size(desktop_plugin);
+      calendar_update_view(desktop_plugin);
+    }
+    else if(!desktop_plugin->time_out_handler)
+    {
+      register_time_out_handler(desktop_plugin);
     }
   }
   else if(desktop_plugin->time_out_handler)
@@ -305,7 +329,7 @@ static void
 cal_home_show_calendar(DBusGProxy* proxy,
 		       int type)
 {
-  
+
   time_t tt = time(NULL); 
   int msgID = tt; 
   const char* text = "DUMMY";  
@@ -320,6 +344,13 @@ cal_home_show_calendar(DBusGProxy* proxy,
   }  
 }
 
+static gboolean
+cal_home_button_press(GtkWidget* widget, GdkEventButton *event, CalHomePlugin* desktop_plugin)
+{
+  desktop_plugin->press_time = time_get_time();
+  return TRUE;
+}
+
 static gboolean
 cal_home_button_release(GtkWidget* widget, GdkEventButton *event, CalHomePlugin* desktop_plugin)
 {
@@ -339,8 +370,16 @@ cal_home_button_release_show_agenda(GtkWidget* widget, GdkEventButton *event, Ca
 static gboolean
 cal_home_button_release_show_week_view(GtkWidget* widget, GdkEventButton *event, CalHomePlugin* desktop_plugin)
 {
-  cal_home_show_calendar(desktop_plugin->dbus_calendar_proxy,
-			 2);
+  int t = time_get_time() - desktop_plugin->press_time;
+
+  if(t<1)
+    cal_home_show_calendar(desktop_plugin->dbus_calendar_proxy,
+			   2);
+  else
+  {
+    calendar_update_view(desktop_plugin);
+    hildon_banner_show_information(NULL, NULL, "Calendar Widget updated");    
+  }
   return TRUE;
 }
 
@@ -413,10 +452,17 @@ build_ui(CalHomePlugin *desktop_plugin)
 		   G_CALLBACK(cal_home_button_release_show_week_view), 
 		   desktop_plugin);
 
+  g_signal_connect(GTK_CONTAINER(week_day_box), 
+		   "button-press-event", 
+		   G_CALLBACK(cal_home_button_press), 
+		   desktop_plugin);
+
   g_signal_connect(GTK_CONTAINER(month_box), "button-release-event", 
 		   G_CALLBACK(cal_home_button_release_show_month_view), 
 		   desktop_plugin);
 
+
+
   gtk_box_pack_start(GTK_BOX(box), title, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), desktop_plugin->event_list_container, FALSE, FALSE, 0);
   gtk_container_add(GTK_CONTAINER(contents), box);
@@ -433,9 +479,7 @@ cal_realize(GtkWidget* widget)
   CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(widget);
   desktop_plugin->iD = hd_home_plugin_item_get_applet_id (HD_HOME_PLUGIN_ITEM (widget));
   cal_home_read_settings(desktop_plugin);
-  render_day_on_title(desktop_plugin);
-  cal_home_load_db_events(desktop_plugin);
-  cal_widget_resize_for_cal_size(desktop_plugin);
+  calendar_update_view(desktop_plugin);
   GTK_WIDGET_CLASS(cal_home_plugin_parent_class)->realize(widget);
 }
 
@@ -469,6 +513,16 @@ cal_render_background_auto(cairo_t* cr, CalHomePlugin* desktop_plugin)
     cairo_fill(cr);
     cairo_translate(cr, 0, -64);
   }
+  if(desktop_plugin->today_events > 0 &&
+     desktop_plugin->today_events < row_count)
+  {
+    int today_line_y = desktop_plugin->today_events*CAL_WIDGET_ENTRY_ROW_HEIGHT+CAL_WIDGET_TITLE_HEIGHT;
+    cairo_set_line_width(cr, 1.5);
+    cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
+    cairo_move_to(cr, 8, today_line_y);
+    cairo_line_to(cr, CAL_WIDGET_WIDTH-8, today_line_y);
+    cairo_stroke(cr);
+  }
 }
 
 static gboolean
@@ -481,7 +535,6 @@ cal_expose(GtkWidget* widget, GdkEventExpose *event)
   cairo_rectangle(cr, 0, 0, CAL_WIDGET_WIDTH, 64);
   cairo_fill(cr);
   cal_render_background_auto(cr, desktop_plugin);
-
   cairo_destroy(cr);
   return GTK_WIDGET_CLASS(cal_home_plugin_parent_class)->expose_event(widget, event);
 }
@@ -493,20 +546,14 @@ register_gconf_notify_on_timeformat_changed(CalHomePlugin* desktop_plugin)
   client = gconf_client_get_default();
   if(!GCONF_IS_CLIENT(client))
   {
-    desktop_plugin->gconf_notify_handler =0;
+    desktop_plugin->gconf_notify_handler = 0;
     return;
   }
-  GError* error = NULL;
+  gconf_client_add_dir(client, GCONF_TIME_FORMAT_PATH,
+		       GCONF_CLIENT_PRELOAD_NONE,
+		       NULL);
   guint handle_id = gconf_client_notify_add(client, GCONF_TIME_FORMAT_PATH,
-					    handle_time_format_changed, desktop_plugin, NULL, &error);
-  if(error==NULL)
-  {
-    desktop_plugin->gconf_notify_handler = handle_id;
-  }
-  else
-  {
-    g_error_free(error);
-  }
+					    handle_time_format_changed, desktop_plugin, NULL, NULL);
   g_object_unref(client);
 }
 
@@ -585,6 +632,10 @@ cal_load_original_bg(CalHomePlugin* desktop_plugin)
 static void
 cal_home_plugin_init(CalHomePlugin *desktop_plugin)
 {
+  struct tm update_date;
+  cal_home_time_today_midnight(&update_date);
+  update_date.tm_mday++;
+  
   cal_load_original_bg(desktop_plugin);
   
   desktop_plugin->event_list = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF);
@@ -595,10 +646,13 @@ cal_home_plugin_init(CalHomePlugin *desktop_plugin)
   desktop_plugin->launch_view = 1;
   desktop_plugin->cal_time_period = CAL_TIME_PERIOD_12MONTH;
   desktop_plugin->cal_row_count = MEDIUM_ROW_COUNT;
-  desktop_plugin->has_24_time_format = cal_24_time_format();
+  desktop_plugin->today_events = 0;
+  desktop_plugin->time_to_update = mktime(&update_date);
+  desktop_plugin->press_time = 0;
   desktop_plugin->update_view = TRUE;
-
+  desktop_plugin->has_24_time_format = cal_24_time_format();
   GtkWidget *contents = build_ui(desktop_plugin);
+
   gtk_window_set_default_size (GTK_WINDOW (desktop_plugin), CAL_WIDGET_WIDTH, 252);
 
   cal_home_init_default_calendar_list_setting(desktop_plugin);
@@ -610,8 +664,6 @@ cal_home_plugin_init(CalHomePlugin *desktop_plugin)
   g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(cal_show_settings_dialog), NULL);
   g_signal_connect(desktop_plugin, "notify::is-on-current-desktop", G_CALLBACK(cal_on_current_desktop), NULL);
   g_signal_connect(GTK_CONTAINER(contents), "button-release-event", G_CALLBACK(cal_home_button_release), desktop_plugin);
-  //  g_signal_connect(G_OBJECT(contents), "expose_event", G_CALLBACK(cal_expose), desktop_plugin);
-  register_time_out_handler(desktop_plugin);
   register_gconf_notify_on_timeformat_changed(desktop_plugin);
   register_dbus_signal_on_time_changed(desktop_plugin);
   register_dbus_signal_on_db_changed(desktop_plugin);
@@ -621,6 +673,10 @@ static void
 cal_plugin_finalize(GObject *object)
 {
   CalHomePlugin *desktop_plugin = CAL_HOME_PLUGIN(object);
+
+  if(desktop_plugin->visible_cals)
+    g_free(desktop_plugin->visible_cals);
+
   if(desktop_plugin->time_out_handler)
   {
     g_source_remove(desktop_plugin->time_out_handler);
@@ -635,8 +691,12 @@ cal_plugin_finalize(GObject *object)
       gconf_client_notify_remove(client,
 				 desktop_plugin->gconf_notify_handler);
       desktop_plugin->gconf_notify_handler = 0;
+      gconf_client_remove_dir(client, GCONF_TIME_FORMAT_PATH,
+			   NULL);
+
     }
   }
+
   if(desktop_plugin->dbus_clockd_proxy)
     g_object_unref(desktop_plugin->dbus_clockd_proxy);
 
@@ -650,7 +710,7 @@ cal_plugin_finalize(GObject *object)
   dbus_g_connection_unref(desktop_plugin->dbus_session_conn);
   g_free(desktop_plugin->iD);
   g_object_unref(desktop_plugin->applet_bg);
-  G_OBJECT_CLASS (cal_home_plugin_parent_class)->finalize (object);
+  G_OBJECT_CLASS(cal_home_plugin_parent_class)->finalize (object);
 }
 
 static void
