@@ -74,6 +74,9 @@ struct _CalHomePlugin
   gboolean update_view;
 
   gint cal_row_count;
+  gint today_events;
+  time_t time_to_update;
+  gint press_time;
   CalTimePeriod cal_time_period;
   GtkListStore* event_list;
   GtkWidget* day_label;
@@ -92,5 +95,8 @@ struct _CalHomePluginClass
 GType cal_home_plugin_get_type(void);
 
 G_END_DECLS
+gboolean
+cal_update_time(gpointer data);
+
 
 #endif
